//=============================================================================
// LostSoul.
//=============================================================================
class lostsoulpawn expands ScriptedPawn;

/*
#exec MESH IMPORT MESH=lostsoulpawn ANIVFILE=MODELS\lostsoulpawn_a.3d DATAFILE=MODELS\lostsoulpawn_d.3d X=0 Y=0 Z=0
#exec MESH ORIGIN MESH=lostsoulpawn X=0 Y=0 Z=0

#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=All STARTFRAME=0 NUMFRAMES=26
#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=Still STARTFRAME=0 NUMFRAMES=1
#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=Idle STARTFRAME=0 NUMFRAMES=7
#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=BiteAttack STARTFRAME=6 NUMFRAMES=9
#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=Flying STARTFRAME=15 NUMFRAMES=6
#exec MESH SEQUENCE MESH=lostsoulpawn SEQ=Acquire STARTFRAME=20 NUMFRAMES=6

#exec MESHMAP NEW MESHMAP=lostsoulpawn MESH=lostsoulpawn
#exec MESHMAP SCALE MESHMAP=lostsoulpawn X=0.1 Y=0.1 Z=0.2

#exec TEXTURE IMPORT NAME=lostsoul FILE=lostsoul.pcx GROUP=Skins

#exec MESHMAP SETTEXTURE MESHMAP=lostsoulpawn NUM=1 TEXTURE=lostsoul
#exec MESHMAP SETTEXTURE MESHMAP=lostsoulpawn NUM=2 TEXTURE=lostsoul

#exec AUDIO IMPORT FILE="lsattack1.wav" NAME="lsattack1" GROUP="LostSoul"
#exec AUDIO IMPORT FILE="lssight1.wav" NAME="lssight1" GROUP="LostSoul"
#exec AUDIO IMPORT FILE="lsdeath1.wav" NAME="lsdeath1" GROUP="LostSoul"
#exec AUDIO IMPORT FILE="lsidle1.wav" NAME="lsidle1" GROUP="LostSoul"
#exec AUDIO IMPORT FILE="lspain1.wav" NAME="lspain1" GROUP="LostSoul"
#exec AUDIO IMPORT FILE="lspain2.wav" NAME="lspain2" GROUP="LostSoul"

*/
//-----------------------------------------------------------------------------
// Manta variables.

// Attack damage.
var() byte
	BiteDamage;
var bool bAttackBump;
var bool bAvoidHit;
var(Sounds) sound Bite;
var(Other) class<actor> ExplodeDeathType;

//-----------------------------------------------------------------------------
// Manta functions.

/* PreSetMovement()
default for walking creature.  Re-implement in subclass
for swimming/flying capability
*/

function PreBeginPlay()
{
	Super.PreBeginPlay();
	if ( skill <= 1 )
		Health = 0.6 * Health;
	if ( skill == 0 )
		AttitudeToPlayer = ATTITUDE_Ignore;
}

function PreSetMovement()
{
	MaxDesiredSpeed = 0.6 + 0.13 * skill;
	bCanJump = true;
	bCanWalk = true;
	bCanSwim = true;
	bCanFly = true;
	MinHitWall = -0.6;
	if (Intelligence > BRAINS_Reptile)
		bCanOpenDoors = true;
	if (Intelligence == BRAINS_Human)
		bCanDoSpecial = true;
}

function SetMovementPhysics()
{
	if (Region.Zone.bWaterZone)
		SetPhysics(PHYS_Swimming);
	else
		SetPhysics(PHYS_Flying);
}

function PlayWaiting()
	{
	LoopAnim('Still', 1.0 + 1.0 * FRand());
	}

function PlayPatrolStop()
	{
	LoopAnim('Still', 1.0);
	}

function PlayWaitingAmbush()
	{
	PlayWaiting();
	}

function PlayChallenge()
{
	PlayAnim('Still', 1.0, 1.0);
}

function TweenToFighter(float tweentime)
{
	TweenAnim('Still', tweentime);
}

function TweenToRunning(float tweentime)
{
	if ( (AnimSequence != 'Still') || !bAnimLoop )
		TweenAnim('Still', tweentime);
}

function TweenToWalking(float tweentime)
{
	if ( (AnimSequence != 'Still') || !bAnimLoop )
		TweenAnim('Still', tweentime);
}

function TweenToWaiting(float tweentime)
{
	PlayAnim('Still', 0.2 + 0.8 * FRand());
	SetPhysics(PHYS_Falling);
}

function TweenToPatrolStop(float tweentime)
{
	TweenAnim('Still', tweentime);
}

function PlayRunning()
{
	LoopAnim('Still', -1.0/AirSpeed,, 1.0);
}

function PlayWalking()
{
	LoopAnim('Still', -1.0/AirSpeed,, 1.0);
}

function PlayThreatening()
{
	if ( FRand() < 1.0 )
	{
		PlayAnim('Still', 1.0);
	}
	else
		LoopAnim('Still', 1.0);
}

function PlayTurning()
{
	if (FRand() < 1.0)
		TweenAnim('Still', 1.0);
	else
	{
		LoopAnim('Still', -1.0/AirSpeed,, 0.4);
	}
}

function PlayDying(name DamageType, vector HitLocation)
{
	spawn(ExplodeDeathType);
	PlaySound(Die, SLOT_Talk, 4 * TransientSoundVolume);
	Destroy();
}

function PlayTakeHit(float tweentime, vector HitLoc, int Damage)
{
	TweenAnim('TakeHit', tweentime);
}

function TweenToFalling()
{
	TweenAnim('Still', 1.0);
}

function PlayInAir()
{
	LoopAnim('Still');
}

function PlayLanded(float impactVel)
{
	PlayAnim('Still');
}


function PlayVictoryDance()
{
	PlayAnim('Acquire', 0.6, 0.1);
	PlaySound(Acquire, SLOT_Talk);
}

function PlayMeleeAttack()
{
	local vector adjust;
	adjust = vect(0,0,0.8) * Target.CollisionHeight;
	Acceleration = AccelRate * Normal(Target.Location - Location + adjust);
	Enable('Bump');
	bAttackBump = false;
	//log("Start Melee Attack");
	if (FRand() < 0.5)
		{
		PlayAnim('BiteAttack');
		PlaySound(Bite, SLOT_Interact);
 		}
	else
	{
 		PlayAnim('BiteAttack');
 		PlaySound(Bite, SLOT_Interact);
 	}
 }

state MeleeAttack
{
ignores SeePlayer, HearNoise;

	singular function Bump(actor Other)
	{
		Disable('Bump');
		if (AnimSequence == 'BiteAttack')
			MeleeDamageTarget(BiteDamage, (BiteDamage * 1000.0 * Normal(Target.Location - Location)));
		bAttackBump = true;
		Velocity *= -0.5;
		Acceleration *= -1;
		if (Acceleration.Z < 0)
			Acceleration.Z *= -1;
	}

	function KeepAttacking()
	{
		if (Target == None)
			GotoState('Attacking');
		else if ( (Pawn(Target) != None) && (Pawn(Target).Health == 0) )
			GotoState('Attacking');
		else if (bAttackBump)
		{
			SetTimer(TimeBetweenAttacks, false);
			GotoState('TacticalMove', 'NoCharge');
		}
	}
}

state Charging
{
ignores SeePlayer, HearNoise;

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
							Vector momentum, name damageType)
	{
		Global.TakeDamage(Damage, instigatedBy, hitlocation, momentum, damageType);
		if ( health <= 0 )
			return;
		if (NextState == 'TakeHit')
		{
			if (AttitudeTo(Enemy) == ATTITUDE_Fear)
			{
				NextState = 'Retreating';
				NextLabel = 'Begin';
			}
			else if ( (FRand() < 3 * Damage/Default.Health) && ((Damage > 0.5 * Health) || (VSize(Location - Enemy.Location) > 150)) )
			{
				bAvoidHit = true;
				NextState = 'TacticalMove';
				NextLabel = 'NoCharge';
			}
			else
			{
				NextState = 'Charging';
				NextLabel = 'TakeHit';
			}
			GotoState('TakeHit');
		}
	}
}

state TacticalMove
{
ignores SeePlayer, HearNoise;

	function PickDestination(bool bNoCharge)
	{
		local vector pick, pickdir, enemydir,Y, minDest;
		local actor HitActor;
		local vector HitLocation, HitNormal, collSpec;
		local float Aggression, enemydist, minDist, strafeSize, MaxMove;

		if ( bAvoidHit && (FRand() < 0.7) )
			MaxMove = 300;
		else
			MaxMove = 600;

		bAvoidHit = false;
		enemyDist = VSize(Location - Enemy.Location);
		Aggression = 2 * FRand() - 1.0;

		if (enemyDist < CollisionRadius + Enemy.CollisionRadius + 2 * MeleeRange)
			Aggression = FMin(0.0, Aggression);
		else if (enemyDist > FMax(VSize(OldLocation - Enemy.OldLocation), 240))
			Aggression = FMax(0.0, Aggression);

		enemydir = (Enemy.Location - Location)/enemyDist;
		minDist = FMin(160.0, 5*CollisionRadius);
		Y = (enemydir Cross vect(0,0,1));
		strafeSize = FMin(0.8, (2 * Abs(Aggression) * FRand() - 0.2));
		if (Aggression <= 0)
			strafeSize *= -1;
		enemydir = enemydir * strafeSize;
		if (FRand() < 0.8)
			enemydir.Z = 1.5;
		else
			enemydir.Z = FMax(0,enemydir.Z);

		strafeSize = FMax(0.0, 1 - Abs(strafeSize));
		pickdir = strafeSize * Y;
		pick = Location + (pickdir + enemydir) * (minDist + MaxMove * FRand());
		pick.Z = Location.Z + 60 + 0.65 * MaxMove * FRand();
		minDest = Location + minDist * Normal(pick - location);
		collSpec.X = CollisionRadius;
		collSpec.Y = CollisionRadius;
		collSpec.Z = CollisionHeight;

		HitActor = Trace(HitLocation, HitNormal, minDest, Location, false, collSpec);
		if ( HitActor == None )
		{
			Destination = pick;
			return;
		}
		pick = Location + (enemydir - pickdir) * (minDist + MaxMove * FRand());
		pick.Z = Location.Z + 60 + 0.5 * MaxMove * FRand();
		minDest = Location + minDist * Normal(pick - location);
		HitActor = Trace(HitLocation, HitNormal, minDest, Location, false, collSpec);
		if ( HitActor == None )
		{
			Destination = pick;
			return;
		}

		pick = Location - enemydir * (minDist + MaxMove * FRand());
		pick.Z = Location.Z + 0.5 * MaxMove * FRand();
		minDest = Location + Normal(pick - Location) * minDist;
		HitActor = Trace(HitLocation, HitNormal, minDest, Location, false, collSpec);
		if ( HitActor == None )
		{
			Destination = pick;
			return;
		}

		if ( !bNoCharge && (enemyDist > 120) )
			GotoState('Charging');

		pick = MaxMove * FRand() * VRand();
		pick.Z = FMin(Location.Z, pick.Z);
		Destination = pick;
	}



	function Bump(Actor Other)
	{
		Disable('Bump');
		if (bAttackBump == true)
			bAttackBump = false;
		else if (Other == Enemy)
			{
			bReadyToAttack = true;
			Target = Enemy;
			GotoState('MeleeAttack');
			}
		else if (Enemy.Health <= 0)
			GotoState('Attacking');
	}
}

function eAttitude AttitudeToCreature( pawn other )
{
	if( Other.IsA('PlayerPawn') )
		return ATTITUDE_Hate;
	else if( Other.IsA('lostsoulpawn') )
		return ATTITUDE_Friendly;
	else if( Other.IsA('Gasbag') && !Other.IsA('Manta') )
		return ATTITUDE_Hate;
	else
		return ATTITUDE_Ignore;
}


defaultproperties
{
     BiteDamage=35
     Bite=Sound'MUPawns.lostsoul.lsattack1'
     ExplodeDeathType=Class'UnrealShare.FlameExplosion'
     Acquire=Sound'MUPawns.lostsoul.lssight1'
     Roam=Sound'MUPawns.lostsoul.lsidle1'
     Health=150
     HitSound1=Sound'MUPawns.lostsoul.lspain1'
     HitSound2=Sound'MUPawns.lostsoul.lspain2'
     Die=Sound'MUPawns.lostsoul.lsdeath1'
     MenuName="Lost Soul"
     DrawType=DT_Mesh
     Mesh=LodMesh'chicogargoyle1'
     DrawScale=5.000000
     AmbientSound=Sound'MUPawns.lostsoul.lsidle1'
}
